@@ -1,12 +1,12 @@
 package Config::Any;
-
-use strict;
+# $Id: $
 use warnings;
-
+use strict;
 use Carp;
 use Module::Pluggable::Object ();
+use English qw(-no_match_vars);
 
-our $VERSION = '0.09_02';
+our $VERSION = '0.04';
 
 =head1 NAME
 
@@ -14,21 +14,21 @@ Config::Any - Load configuration from different file formats, transparently
 
 =head1 VERSION
 
-This document describes Config::Any version 0.09_02
+This document describes Config::Any version 0.0.4
 
 =head1 SYNOPSIS
 
     use Config::Any;
 
-    my $cfg = Config::Any->load_stems({stems => \@filepath_stems, ... });
-    # or
-    my $cfg = Config::Any->load_files({files => \@filepaths, ... });
+	my $cfg = Config::Any->load_stems({stems => \@filepath_stems, ... });
+	# or
+	my $cfg = Config::Any->load_files({files => \@filepaths, ... });
 
-    for (@$cfg) {
-        my ($filename, $config) = %$_;
-        $class->config($config);
-        warn "loaded config from file: $filename";
-    }
+	for (@$cfg) {
+		my ($filename, $config) = each %$_;
+		$class->config($config);
+		warn "loaded config from file: $filename";
+	}
 
 =head1 DESCRIPTION
 
@@ -48,11 +48,11 @@ configuration formats.
 
 =cut
 
-=head2 load_files( \%args )
+=head2 load_files( )
 
-    Config::Any->load_files( { files => \@files } );
-    Config::Any->load_files( { files => \@files, filter  => \&filter } );
-    Config::Any->load_files( { files => \@files, use_ext => 1 } );
+    Config::Any->load_files({files => \@files]});
+    Config::Any->load_files({files => \@files, filter  => \&filter});
+    Config::Any->load_files({files => \@files, use_ext => 1});
 
 C<load_files()> attempts to load configuration from the list of files passed in
 the C<files> parameter, if the file exists.
@@ -69,36 +69,27 @@ be aware that you will lose flexibility -- for example, a file called C<myapp.cf
 containing YAML data will not be offered to the YAML plugin, whereas C<myapp.yml>
 or C<myapp.yaml> would be.
 
-C<load_files()> also supports a 'force_plugins' parameter, whose value should be an
-arrayref of plugin names like C<Config::Any::INI>. Its intended use is to allow the use 
-of a non-standard file extension while forcing it to be offered to a particular parser.
-It is not compatible with 'use_ext'. 
-
-You can supply a C<driver_args> hashref to pass special options to a particular
-parser object. Example:
-
-    Config::Any->load_files( { files => \@files, driver_args => {
-        General => { -LowerCaseNames => 1 }
-    } )
-
 =cut
 
 sub load_files {
-    my ( $class, $args ) = @_;
-
-    unless ( $args && exists $args->{ files } ) {
-        warn "No files specified!";
+    my ($class, $args) = @_;
+    return unless defined $args;
+    unless (exists $args->{files}) {
+        warn "no files specified";
         return;
     }
 
-    return $class->_load( $args );
+    my $files = [ grep { -f $_ } @{$args->{files}} ];
+    my $filter_cb = delete $args->{filter};
+    my $use_ext   = delete $args->{use_ext};
+    return $class->_load($files, $filter_cb, $use_ext);
 }
 
-=head2 load_stems( \%args )
+=head2 load_stems( )
 
-    Config::Any->load_stems( { stems => \@stems } );
-    Config::Any->load_stems( { stems => \@stems, filter  => \&filter } );
-    Config::Any->load_stems( { stems => \@stems, use_ext => 1 } );
+    Config::Any->load_stems({stems => \@stems]});
+    Config::Any->load_stems({stems => \@stems, filter  => \&filter});
+    Config::Any->load_stems({stems => \@stems, use_ext => 1});
 
 C<load_stems()> attempts to load configuration from a list of files which it generates
 by combining the filename stems list passed in the C<stems> parameter with the 
@@ -110,94 +101,74 @@ parameters. Please read the C<load_files()> documentation before using this meth
 =cut
 
 sub load_stems {
-    my ( $class, $args ) = @_;
-
-    unless ( $args && exists $args->{ stems } ) {
-        warn "No stems specified!";
+    my ($class, $args) = @_;
+    return unless defined $args;
+    unless (exists $args->{stems}) {
+        warn "no stems specified";
         return;
     }
-
-    my $stems = delete $args->{ stems };
+        
+    my $filter_cb = delete $args->{filter};
+    my $use_ext   = delete $args->{use_ext};
+    my $stems = $args->{stems};
     my @files;
-    for my $s ( @$stems ) {
-        for my $ext ( $class->extensions ) {
-            push @files, "$s.$ext";
+    STEM:
+    for my $s (@$stems) {
+        EXT:
+        for my $ext ($class->extensions) {
+            my $file = "$s.$ext";
+            next EXT unless -f $file;
+            push @files, $file;
+            last EXT;
         }
     }
-
-    $args->{ files } = \@files;
-    return $class->_load( $args );
+    return $class->_load(\@files, $filter_cb, $use_ext);
 }
 
+# this is where we do the real work
+# it's a private class-method because users should use the interface described
+# in the POD.
 sub _load {
-    my ( $class, $args ) = @_;
-    croak "_load requires a arrayref of file paths" unless $args->{ files };
+    my ($class, $files_ref, $filter_cb, $use_ext) = @_;
+    croak "_load requires a arrayref of file paths" unless defined $files_ref;
 
-    my $force = defined $args->{ force_plugins };
-    if ( !$force and !defined $args->{ use_ext } ) {
-        warn
-            "use_ext argument was not explicitly set, as of 0.09, this is true by default";
-        $args->{ use_ext } = 1;
-    }
+    my $final_configs       = [];
+    my $originally_loaded   = {};
+	my %files = map { $_ => 1 } @$files_ref;
 
-    # figure out what plugins we're using
-    my @plugins = $force ? @{ $args->{ force_plugins } } : $class->plugins;
-
-    # map extensions if we have to
-    my ( %extension_lut, $extension_re );
-    my $use_ext_lut = !$force && $args->{ use_ext };
-    if ( $use_ext_lut ) {
-        for my $plugin ( @plugins ) {
-            $extension_lut{ $_ } = $plugin for $plugin->extensions;
-        }
-
-        $extension_re = join( '|', keys %extension_lut );
-    }
-
-    # map args to plugins
-    my $base_class = __PACKAGE__;
-    my %loader_args;
-    for my $plugin ( @plugins ) {
-        $plugin =~ m{^$base_class\::(.+)};
-        $loader_args{ $plugin } = $args->{ driver_args }->{ $1 } || {};
-    }
-
-    my @results;
-
-    for my $filename ( @{ $args->{ files } } ) {
-
-        # don't even bother if it's not there
-        next unless -f $filename;
-
-        my @try_plugins = @plugins;
-
-        if ( $use_ext_lut ) {
-            $filename =~ m{\.($extension_re)\z};
-            next unless $1;
-            @try_plugins = $extension_lut{ $1 };
-        }
-
-        for my $loader ( @try_plugins ) {
-            next unless $loader->is_supported;
-            my @configs
-                = eval { $loader->load( $filename, $loader_args{ $loader } ); };
-
-            # fatal error if we used extension matching
-            croak "Error parsing file: $filename" if $@ and $use_ext_lut;
-            next if $@ or !@configs;
-
-            # post-process config with a filter callback
-            if ( $args->{ filter } ) {
-                $args->{ filter }->( $_ ) for @configs;
+    for my $loader ( $class->plugins ) {
+		last unless keys %files;
+#	warn "loader: $loader\n";
+        my %ext = map { $_ => 1 } $loader->extensions;
+        FILE:
+        for my $filename (keys %files) {
+            if (defined $use_ext) {
+#			warn "using file extension to decide which loader to use for file $filename\n";
+				my $matched_ext = 0;
+                EXT:
+                for my $e (keys %ext) {
+#				warn "trying ext $e\n";
+                    next EXT  unless $filename =~ m{ \. $e \z }xms; 
+#					warn "filename $filename matched extension $e\n";
+                    next FILE unless exists $ext{$e};
+					$matched_ext = 1;
+                }
+				next FILE unless $matched_ext;
             }
 
-            push @results,
-                { $filename => @configs == 1 ? $configs[ 0 ] : \@configs };
-            last;
+            my $config;
+			eval {
+				$config = $loader->load( $filename );
+			};
+			next if $EVAL_ERROR;
+            next if !$config;
+			delete $files{$filename};
+#			warn "loader $loader loaded file $filename\n";
+            $filter_cb->( $config ) if defined $filter_cb;
+            push @$final_configs, { $filename => $config };
         }
     }
-
-    return \@results;
+    $final_configs;
 }
 
 =head2 finder( )
@@ -210,12 +181,12 @@ more information.
 =cut
 
 sub finder {
-    my $class  = shift;
+    my $class = shift;
     my $finder = Module::Pluggable::Object->new(
         search_path => [ __PACKAGE__ ],
         require     => 1
     );
-    return $finder;
+    $finder;
 }
 
 =head2 plugins( )
@@ -241,14 +212,14 @@ parameter to those methods.
 sub extensions {
     my $class = shift;
     my @ext = map { $_->extensions } $class->plugins;
-    return wantarray ? @ext : \@ext;
+	return wantarray ? @ext : [@ext];
 }
 
 =head1 DIAGNOSTICS
 
 =over
 
-=item C<No files specified!> or C<No stems specified!>
+=item C<no files specified> or C<no stems specified>
 
 The C<load_files()> and C<load_stems()> methods will issue this warning if
 called with an empty list of files/stems to load.
@@ -292,7 +263,7 @@ L<http://rt.cpan.org>.
 
 =head1 AUTHOR
 
-Joel Bernstein  E<lt>rataxis@cpan.orgE<gt>
+Joel Bernstein  C<< <rataxis@cpan.org> >>
 
 =head1 CONTRIBUTORS
 
@@ -302,12 +273,9 @@ module by Brian Cassidy C<< <bricas@cpan.org> >>.
 
 With ideas and support from Matt S Trout C<< <mst@shadowcatsystems.co.uk> >>.
 
-Further enhancements suggested by Evan Kaufman C<< <evank@cpan.org> >>.
-
 =head1 LICENCE AND COPYRIGHT
 
 Copyright (c) 2006, Portugal Telecom C<< http://www.sapo.pt/ >>. All rights reserved.
-Portions copyright 2007, Joel Bernstein C<< <rataxis@cpan.org> >>.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
@@ -342,4 +310,5 @@ L<Catalyst::Plugin::ConfigLoader|Catalyst::Plugin::ConfigLoader>
 
 =cut
 
-"Drink more beer";
+1; # Magic true value required at end of module
+
